SystemOrganization addCategory: #'Constraints-Tests'!
SystemOrganization addCategory: #'Constraints-Core'!

!ContextPart methodsFor: '*Constraints' stamp: 'mg 4/16/2014 16:05'!
doPrimitive: primitiveIndex method: meth receiver: receiver args: arguments 
	"Simulate a primitive method whose index is primitiveIndex.  The
	 simulated receiver and arguments are given as arguments to this message.
	 If successful, push result and return resuming context, else ^ PrimitiveFailToken.
	 Any primitive which provokes execution needs to be intercepted and simulated
	 to avoid execution running away."
	<layer: #ccm>
	| value |
	"Judicious use of primitive 19 (a null primitive that doesn't do anything) prevents
	 the debugger from entering various run-away activities such as spawning a new
	 process, etc.  Injudicious use results in the debugger not being able to debug
	 interesting code, such as the debugger itself.  hence use primtiive 19 with care :-)"
	"SystemNavigation new browseAllSelect: [:m| m primitive = 19]"
	primitiveIndex = 19 ifTrue:
		[ToolSet 
			debugContext: self
			label:'Code simulation error'
			contents: nil].

	"ContextPart>>blockCopy:; simulated to get startpc right"
	(primitiveIndex = 80 and: [(self objectClass: receiver) includesBehavior: ContextPart]) 
		ifTrue: [^self push: ((BlockContext newForMethod: receiver method)
						home: receiver home
						startpc: pc + 2
						nargs: (arguments at: 1))].
	(primitiveIndex = 81 and: [(self objectClass: receiver) == BlockContext]) "BlockContext>>value[:value:...]"
		ifTrue: [^receiver pushArgs: arguments from: self].
	(primitiveIndex = 82 and: [(self objectClass: receiver) == BlockContext]) "BlockContext>>valueWithArguments:"
		ifTrue: [^receiver pushArgs: arguments first from: self].
	primitiveIndex = 83 "afr 9/11/1998 19:50" "Object>>perform:[with:...]"
		ifTrue: [^self send: arguments first
					to: receiver
					with: arguments allButFirst
					super: false].
	primitiveIndex = 84 "afr 9/11/1998 19:50 & eem 8/18/2009 17:04" "Object>>perform:withArguments:"
		ifTrue: [^self send: arguments first
					to: receiver
					with: (arguments at: 2)
					startClass: nil].
	primitiveIndex = 100 "eem 8/18/2009 16:57" "Object>>perform:withArguments:inSuperclass:"
		ifTrue: [^self send: arguments first
					to: receiver
					with: (arguments at: 2)
					startClass: (arguments at: 3)].

	"Mutex>>primitiveEnterCriticalSection
	 Mutex>>primitiveTestAndSetOwnershipOfCriticalSection"
	(primitiveIndex = 186 or: [primitiveIndex = 187]) ifTrue:
		[| active effective |
		 active := Processor activeProcess.
		 effective := active effectiveProcess.
		 "active == effective"
		 value := primitiveIndex = 186
					ifTrue: [receiver primitiveEnterCriticalSectionOnBehalfOf: effective]
					ifFalse: [receiver primitiveTestAndSetOwnershipOfCriticalSectionOnBehalfOf: effective].
		 ^((self objectClass: value) == Array
		    and: [value size = 2
		    and: [value first == PrimitiveFailToken]])
			ifTrue: [value]
			ifFalse: [self push: value]].

	primitiveIndex = 188 ifTrue: "eem 5/27/2008 11:10 Object>>withArgs:executeMethod:"
		[^MethodContext
			sender: self
			receiver: receiver
			method: (arguments at: 2)
			arguments: (arguments at: 1)].

	"Closure primitives"
	(primitiveIndex = 200 and: [self == receiver]) ifTrue:
		"ContextPart>>closureCopy:copiedValues:; simulated to get startpc right"
		[^self push: (BlockClosure
						outerContext: receiver
						startpc: pc + 2
						numArgs: arguments first
						copiedValues: arguments last)].
	((primitiveIndex between: 201 and: 205)			 "BlockClosure>>value[:value:...]"
	or: [primitiveIndex between: 221 and: 222]) ifTrue: "BlockClosure>>valueNoContextSwitch[:]"
		[^receiver simulateValueWithArguments: arguments caller: self].
	primitiveIndex = 206 ifTrue:						"BlockClosure>>valueWithArguments:"
		[^receiver simulateValueWithArguments: arguments first caller: self].

	primitiveIndex = 118 ifTrue: "tryPrimitive:withArgs:; avoid recursing in the VM"
		[(arguments size = 2
		 and: [arguments first isInteger
		 and: [(self objectClass: arguments last) == Array]]) ifFalse:
			[^ContextPart primitiveFailTokenFor: nil].
		 ^self doPrimitive: arguments first method: meth receiver: receiver args: arguments last].

	value := primitiveIndex = 120 "FFI method"
				ifTrue: [(meth literalAt: 1) tryInvokeWithArguments: arguments]
				ifFalse:
					[primitiveIndex = 117 "named primitives"
						ifTrue: [self tryNamedPrimitiveIn: meth for: receiver withArgs: arguments]
						ifFalse:
							[receiver tryPrimitive: primitiveIndex withArgs: arguments]].
	^((self objectClass: value) == Array
	    and: [value size = 2
	    and: [value first == PrimitiveFailToken]])
		ifTrue: [value]
		ifFalse: ["self push: (self valueOrConstraintVariableFor: meth receiver: receiver value: value)"
				self push: value asConstraintVariable]! !

!ContextPart methodsFor: '*Constraints' stamp: 'mg 4/16/2014 14:49'!
send: selector super: superFlag numArgs: numArgs
	"Simulate the action of bytecodes that send a message with selector, 
	selector. The argument, superFlag, tells whether the receiver of the 
	message was specified with 'super' in the source method. The arguments 
	of the message are found in the top numArgs locations on the stack and 
	the receiver just below them."
	<layer: #ccm>
	| receiver arguments |
	Transcript show: 'layer method send '; show: selector; cr.
	arguments := Array new: numArgs.
	numArgs to: 1 by: -1 do: [ :i | arguments at: i put: self pop].
	receiver := self pop.
	selector = '=' 
		ifTrue: [Transcript 
			show: 'send = aruments: '; show: arguments; 
			show: ' receiver: '; show: receiver; 
			cr].
	selector = 'terminateProcess' 
		ifTrue: [Transcript show: 'process is terminating'; cr. ^ nil].
	QuickStep == self ifTrue:
		[QuickStep := nil.
		^self quickSend: selector to: receiver with: arguments super: superFlag].
	^self send: selector to: receiver with: arguments super: superFlag! !

!ContextPart methodsFor: '*Constraints' stamp: 'mg 4/16/2014 16:02'!
valueOrConstraintVariableFor: method receiver: receiver value: value
	<layer: #ccm>
	receiver class category =  #'Constraints-Core' 
		ifTrue: ["Don't change values inside the ConstraintSolver package" ^ value].
	method isReturnField 
		ifTrue: [Transcript show: 'found instVarAccesor of instance '; show: receiver; cr.
			^ value asConstraintVariable].
	^ value! !

!BlockClosure methodsFor: '*Constraints' stamp: 'mg 4/15/2014 13:45'!
alwaysTrue
	"This method constructs constraints for all solvable constraints inside this blockClosure."
	| newProcess |
	newProcess := self newTerminatingProcess.
	#ccm withLayerDo: [[newProcess step] doWhileFalse: [newProcess suspendedContext isNil]].
	newProcess terminate.! !

!BlockClosure methodsFor: '*Constraints' stamp: 'mg 4/15/2014 13:33'!
newTerminatingProcess
	"Answer a Process running the code in the receiver. The process is not 
	scheduled."
	<primitive: 19> "Simulation guard"
	^Process
		forContext: 
			[self value. self terminateProcess.] asContext
		priority: Processor activePriority! !

!BlockClosure methodsFor: '*Constraints' stamp: 'mg 4/15/2014 13:33'!
terminateProcess

	^ self! !

!ClStrength methodsFor: '*Constraints' stamp: 'mg 4/16/2014 15:32'!
isMedium

	^ self == MediumStrength! !

!ClStrength methodsFor: '*Constraints' stamp: 'mg 4/16/2014 15:32'!
isStrong

	^ self == StrongStrength! !

!ClStrength methodsFor: '*Constraints' stamp: 'mg 4/16/2014 15:32'!
isWeak

	^ self == WeakStrength! !

TestCase subclass: #ConstraintsTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Constraints-Tests'!

!ConstraintsTests methodsFor: 'testing' stamp: 'mg 4/15/2014 14:12'!
testNoException

	self shouldnt: [[1 = 1] alwaysTrue] raise: Exception.! !

!ConstraintsTests methodsFor: 'testing' stamp: 'mg 4/16/2014 12:02'!
testPriorityOrder

	| p1 p2 p3 p4 |
	p1 := ConstraintPriority required.
	p2 := ConstraintPriority strong.
	p3 := ConstraintPriority medium.
	p4 := ConstraintPriority weak.

	self should: [p1 <= p2].
	self should: [p2 <= p3].
	self should: [p3 <= p4].

	self should: [p1 < p2].
	self should: [p2 < p3].
	self should: [p3 < p4].

	self shouldnt: [p1 < p1].! !

!ConstraintsTests methodsFor: 'testing' stamp: 'mg 4/16/2014 16:29'!
testSimpleAssignmentConstraint

	| ex |
	self fail.
	ex := ExampleClass new.
	[ex a = 42] alwaysTrue.
	self should: [ex a = 42].! !

!ConstraintsTests methodsFor: 'testing' stamp: 'mg 4/15/2014 14:24'!
testUnsatisfiableConstraints

	self should: [[1 = 2] alwaysTrue] raise: Exception.! !

TestCase subclass: #VariableTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Constraints-Tests'!

!VariableTests methodsFor: 'testing' stamp: 'mg 4/17/2014 13:15'!
testCassowaryVariableEqualityOperator

	| const2 const5 var17 var23 varNil varNil2 varTest |
	var17 := ConstraintVariable newCassowaryVariableFor: 17.
	var23 := 23 asConstraintVariable.
	varTest := 'test' asConstraintVariable.
	varNil := ConstraintCassowaryVariable new.
	varNil2 := ConstraintCassowaryVariable new.

	const2 := 2 asConstantConstraintVariable.
	const5 := 5 asConstantConstraintVariable.

	self should: [var17 isCassowaryVariable].
	self shouldnt: [varTest isCassowaryVariable].

	self should: [const2 = const5] raise: Error.
	self should: [const2 = const2 copy].
	self shouldnt: [var17 = const2] raise: Error.	"var17 value notNil"
	self should: [var17 value = const2 value].
	self shouldnt: [varNil = const2] raise: Error.	"varNil value isNil"	
	self should: [varNil value = const2 value].

	self should: [var17 = var23] raise: Error.	"values different, notNil"
	self should: [var17 = varTest] raise: Error.	"different variable types"
	self shouldnt: [var23 = varNil2] raise: Error. 	"varNil2 value isNil"
	self should: [var23 value = varNil2 value].

	varNil := ConstraintCassowaryVariable new.
	varNil2 := ConstraintCassowaryVariable new.
	self shouldnt: [varNil = varNil2] raise: Error.	"both values are nil"
	self should: [varNil value isNil].
	self should: [varNil2 value isNil].! !

Object subclass: #Constraint
	instanceVariableNames: 'solver priority externalConstraint'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Constraints-Core'!

!Constraint methodsFor: 'as yet unclassified' stamp: 'mg 4/16/2014 12:02'!
< aConstraint

	self priority < aConstraint priority! !

!Constraint methodsFor: 'as yet unclassified' stamp: 'mg 4/16/2014 11:46'!
<= aConstraint

	self priority <= aConstraint priority! !

!Constraint methodsFor: 'accessing' stamp: 'mg 4/16/2014 13:12'!
externalConstraint

	^ externalConstraint! !

!Constraint methodsFor: 'accessing' stamp: 'mg 4/16/2014 13:12'!
externalConstraint: anObject

	externalConstraint := anObject! !

!Constraint methodsFor: 'initialize-release' stamp: 'mg 4/16/2014 11:58'!
initialize

	priority := ConstraintPriority required.! !

!Constraint methodsFor: 'accessing' stamp: 'mg 4/16/2014 13:12'!
priority

	^ priority! !

!Constraint methodsFor: 'accessing' stamp: 'mg 4/16/2014 13:12'!
priority: anObject

	priority := anObject! !

!Constraint methodsFor: 'accessing' stamp: 'mg 4/16/2014 13:12'!
solver

	^ solver! !

!Constraint methodsFor: 'accessing' stamp: 'mg 4/16/2014 16:33'!
solver: anObject

	solver := anObject.
	anObject addConstraint: self.! !

!Constraint methodsFor: 'as yet unclassified' stamp: 'mg 4/16/2014 13:16'!
triggerSolver
	"This method should be called, if this constraint changed, e.g., because a variable changed. It triggers the solver to solve all constraints. The solving process may raise an exception, if the constraints are unsatisfiable."
	self solver solve.! !

Constraint subclass: #EqualityConstraint
	instanceVariableNames: 'variable newValue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Constraints-Core'!

!EqualityConstraint commentStamp: 'mg 4/16/2014 14:23' prior: 0!
An EqualityConstraint is used for changing the value of a ConstraintVariable. Changing the value will add an EqualityConstraint to the solver and try to solve it. It has the form "variable = newValue".

Instance Variables
	newValue		The new value
	variable:	The changed ConstraintVariable
!

!EqualityConstraint class methodsFor: 'as yet unclassified' stamp: 'mg 4/16/2014 14:22'!
newFor: aConstraintVariable value: aValue

	^ self new variable: aConstraintVariable; newValue: aValue! !

!EqualityConstraint methodsFor: 'accessing' stamp: 'mg 4/16/2014 14:21'!
newValue

	^ newValue! !

!EqualityConstraint methodsFor: 'accessing' stamp: 'mg 4/16/2014 14:21'!
newValue: anObject

	newValue := anObject! !

!EqualityConstraint methodsFor: 'accessing' stamp: 'mg 4/16/2014 14:21'!
variable

	^ variable! !

!EqualityConstraint methodsFor: 'accessing' stamp: 'mg 4/16/2014 14:21'!
variable: anObject

	variable := anObject! !

Object subclass: #ConstraintGenericVariable
	instanceVariableNames: 'constraint originalValue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Constraints-Core'!

!ConstraintGenericVariable commentStamp: 'mg 4/16/2014 13:02' prior: 0!
A ConstraintGenericVariable represents all values that are used inside a constraint but does not have a concrete ConstraintVariable.

Instance Variables
	constraint:		The constraint is needed to invalidate and recreate the constraint, if this generic variable changes its value.
	originalValue:	The original object
!

!ConstraintGenericVariable methodsFor: 'accessing' stamp: 'mg 4/16/2014 15:56'!
asConstraintVariable

	^ self! !

!ConstraintGenericVariable methodsFor: 'accessing' stamp: 'mg 4/16/2014 12:52'!
constraint

	^ constraint! !

!ConstraintGenericVariable methodsFor: 'accessing' stamp: 'mg 4/16/2014 12:52'!
constraint: anObject

	constraint := anObject! !

!ConstraintGenericVariable methodsFor: 'accessing' stamp: 'mg 4/16/2014 14:42'!
isCassowaryVariable

	^ false! !

!ConstraintGenericVariable methodsFor: 'accessing' stamp: 'mg 4/17/2014 10:49'!
isConstant
	"In general, constraint variables can be changed."
	^ false! !

!ConstraintGenericVariable methodsFor: 'accessing' stamp: 'mg 4/16/2014 12:52'!
originalValue

	^ originalValue! !

!ConstraintGenericVariable methodsFor: 'accessing' stamp: 'mg 4/16/2014 12:52'!
originalValue: anObject

	originalValue := anObject! !

!ConstraintGenericVariable methodsFor: 'accessing' stamp: 'mg 4/16/2014 11:26'!
suggestValue: aValue
	"This method adds an equality constraint for the new value. If all constraints are still solvable, the value is changed to aValue. Otherwise, the old value does not change and an exception is raised. Generic variables are not supposed to be changed. If so, all corresponding constraints should be recreated."! !

!ConstraintGenericVariable methodsFor: 'accessing' stamp: 'mg 4/17/2014 10:06'!
value
	"Returns the value from the solver variable. A generic variable forwards all messages to the original object."
	^ originalValue! !

ConstraintGenericVariable subclass: #ConstraintVariable
	instanceVariableNames: 'externalVariable'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Constraints-Core'!

!ConstraintVariable commentStamp: 'mg 4/16/2014 13:07' prior: 0!
A ConstraintVariable is a concrete ConstraintVariable that does have a solver object for it.

Instance Variables
	externalVariable:	This is the solver object for the constraint variable.
!

ConstraintVariable subclass: #ConstraintCassowaryVariable
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Constraints-Core'!

!ConstraintCassowaryVariable methodsFor: 'comparing' stamp: 'mg 4/17/2014 13:02'!
= aConstraintVariable

	| extConstraint rhs |
	rhs := aConstraintVariable asConstraintVariable.
	(rhs isCassowaryVariable or: [rhs isConstant])
		ifFalse: [self error: 'Cannot compare different types of constraint variable.'].
	rhs isConstant
		ifTrue: [extConstraint := self externalVariable cnEqual: rhs value]
		ifFalse: ["rhs is a Cassowary variable."
			self value ~= rhs value
				ifTrue: [self error: 'The constraint is unsatisfiable, because the values are different.'].
			extConstraint := self externalVariable cnEqual: rhs externalVariable].
	constraint := Constraint new 
		externalConstraint: extConstraint;
		priority: (ConstraintPriority fromCassowaryStrength: extConstraint strength);
		solver: ConstraintSolver cassowarySolver.
	constraint triggerSolver.
	^ constraint! !

!ConstraintCassowaryVariable methodsFor: 'accessing' stamp: 'mg 4/16/2014 14:42'!
isCassowaryVariable
	
	^ true! !

!ConstraintCassowaryVariable methodsFor: 'accessing' stamp: 'mg 4/17/2014 13:10'!
value

	^ externalVariable value! !

ConstraintVariable subclass: #ConstraintConstantVariable
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Constraints-Core'!

!ConstraintConstantVariable methodsFor: 'comparing' stamp: 'mg 4/17/2014 11:22'!
= aConstraintVariable

	(aConstraintVariable isConstant and: [self value ~= aConstraintVariable value])
		ifTrue: [self error: 'Both variables are constant, but their values are different.']
		ifFalse: [^ true].	"The constraint is always true and therefore should not be added to the solver."
	aConstraintVariable value 
		ifNil: [aConstraintVariable suggestValue: self value].	"The value can be nil, if it is not a constant variable."
	^ aConstraintVariable = self
! !

!ConstraintConstantVariable methodsFor: 'accessing' stamp: 'mg 4/17/2014 11:00'!
asConstantConstraintVariable

	^ self! !

!ConstraintConstantVariable methodsFor: 'initialize-release' stamp: 'mg 4/17/2014 11:15'!
initialize
	"The value of a constant variable should never be nil, because it cannot be changed afterwards."

	originalValue := -1.! !

!ConstraintConstantVariable methodsFor: 'accessing' stamp: 'mg 4/17/2014 10:47'!
isConstant
	"Constant variables are read-only."
	^ true! !

!ConstraintConstantVariable methodsFor: 'accessing' stamp: 'mg 4/17/2014 10:51'!
suggestValue: aValue

	self error: 'Thic constraint variable is read-only.'.! !

!ConstraintVariable class methodsFor: 'instance creation' stamp: 'mg 4/16/2014 14:37'!
newCassowaryVariableFor: aNumber

	| cv extVar |
	cv := ConstraintCassowaryVariable new.
	cv originalValue: aNumber.
	extVar := ClVariable newWith: aNumber asFloat name: aNumber name.
	cv externalVariable: extVar.
	^ cv! !

!ConstraintVariable class methodsFor: 'instance creation' stamp: 'mg 4/17/2014 10:59'!
newConstantVariableFor: aNumber

	| cv extVar |
	cv := ConstraintConstantVariable new.
	cv originalValue: aNumber.
	"maybe the external variable is another one?"
	extVar := ClVariable newWith: aNumber asFloat name: aNumber name.
	cv externalVariable: extVar.
	^ cv! !

!ConstraintVariable methodsFor: 'constraint-construction' stamp: 'mg 4/16/2014 12:52'!
+ aConstraintVariable
	"This method should return a linear expression"
	self originalValue: 2358.! !

!ConstraintVariable methodsFor: 'accessing' stamp: 'mg 4/17/2014 10:54'!
asConstantConstraintVariable

	^ ConstraintConstantVariable new
			constraint: self constraint;
			originalValue: self originalValue;
			externalVariable: self externalVariable! !

!ConstraintVariable methodsFor: 'accessing' stamp: 'mg 4/16/2014 13:21'!
externalVariable

	^ externalVariable! !

!ConstraintVariable methodsFor: 'accessing' stamp: 'mg 4/16/2014 13:21'!
externalVariable: anObject

	externalVariable := anObject! !

!ConstraintVariable methodsFor: 'accessing' stamp: 'mg 4/16/2014 11:19'!
suggestValue: aValue
	"This method adds an equality constraint for the new value. If all constraints are still solvable, the value is changed to aValue. Otherwise, the old value does not change and an exception is raised."! !

!ConstraintVariable methodsFor: 'accessing' stamp: 'mg 4/16/2014 12:55'!
value
	"Returns the value from the solver variable."
	^ originalValue! !

Object subclass: #ConstraintLinearExpression
	instanceVariableNames: 'externalExpression'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Constraints-Core'!

!ConstraintLinearExpression methodsFor: 'accessing' stamp: 'mg 4/16/2014 13:20'!
externalExpression

	^ externalExpression! !

!ConstraintLinearExpression methodsFor: 'accessing' stamp: 'mg 4/16/2014 13:20'!
externalExpression: anObject

	externalExpression := anObject! !

Object subclass: #ConstraintPriority
	instanceVariableNames: 'name val'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Constraints-Core'!

!ConstraintPriority commentStamp: 'mg 4/16/2014 12:05' prior: 0!
A ConstraintPriority is a named priority object. It represents the priority of a constraint.

Instance Variables
	name:	symbolic name for a priority
	val:		a number for the priority to simplify comparisons between priorities 

!

!ConstraintPriority class methodsFor: 'instance creation' stamp: 'mg 4/16/2014 15:34'!
fromCassowaryStrength: aClStrength

	aClStrength isRequired ifTrue: [^ ConstraintPriority required].
	aClStrength isStrong ifTrue: [^ ConstraintPriority strong].
	aClStrength isMedium ifTrue: [^ ConstraintPriority medium].
	aClStrength isWeak ifTrue: [^ ConstraintPriority weak].! !

!ConstraintPriority class methodsFor: 'instance creation' stamp: 'mg 4/16/2014 11:56'!
medium

	^ self new 
		name: 'medium';
		val: 2! !

!ConstraintPriority class methodsFor: 'instance creation' stamp: 'mg 4/16/2014 11:55'!
required

	^ self new 
		name: 'required';
		val: 0! !

!ConstraintPriority class methodsFor: 'instance creation' stamp: 'mg 4/16/2014 11:55'!
strong

	^ self new 
		name: 'strong';
		val: 1! !

!ConstraintPriority class methodsFor: 'instance creation' stamp: 'mg 4/16/2014 11:56'!
weak

	^ self new 
		name: 'weak';
		val: 3! !

!ConstraintPriority methodsFor: 'as yet unclassified' stamp: 'mg 4/16/2014 12:02'!
< aConstraintPriority

	^ self val < aConstraintPriority val! !

!ConstraintPriority methodsFor: 'as yet unclassified' stamp: 'mg 4/16/2014 11:58'!
<= aConstraintPriority

	^ self val <= aConstraintPriority val! !

!ConstraintPriority methodsFor: 'accessing' stamp: 'mg 4/16/2014 11:57'!
name
		
	^ name! !

!ConstraintPriority methodsFor: 'accessing' stamp: 'mg 4/16/2014 11:56'!
name: anObject

	name := anObject! !

!ConstraintPriority methodsFor: 'accessing' stamp: 'mg 4/16/2014 11:56'!
val

	^ val! !

!ConstraintPriority methodsFor: 'accessing' stamp: 'mg 4/16/2014 11:56'!
val: anObject

	val := anObject! !

Object subclass: #ConstraintSolver
	instanceVariableNames: 'externalSolver constraints'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Constraints-Core'!

ConstraintSolver class
	instanceVariableNames: 'CassowarySolver'!

ConstraintSolver class
	instanceVariableNames: 'CassowarySolver'!

!ConstraintSolver class methodsFor: 'instance creation' stamp: 'mg 4/16/2014 15:52'!
cassowarySolver

	CassowarySolver ifNil: [CassowarySolver := self basicNew initialize].
	CassowarySolver externalSolver: ClSimplexSolver new.
	^ CassowarySolver! !

!ConstraintSolver methodsFor: 'as yet unclassified' stamp: 'mg 4/16/2014 13:13'!
addConstraint: aConstraint

	constraints add: aConstraint.
	self externalSolver addConstraint: aConstraint externalConstraint.
	self solve.! !

!ConstraintSolver methodsFor: 'accessing' stamp: 'mg 4/16/2014 13:11'!
constraints

	^ constraints! !

!ConstraintSolver methodsFor: 'accessing' stamp: 'mg 4/16/2014 13:11'!
constraints: anObject

	constraints := anObject! !

!ConstraintSolver methodsFor: 'accessing' stamp: 'mg 4/16/2014 13:11'!
externalSolver

	^ externalSolver! !

!ConstraintSolver methodsFor: 'accessing' stamp: 'mg 4/16/2014 13:11'!
externalSolver: anObject

	externalSolver := anObject! !

!ConstraintSolver methodsFor: 'initialize-release' stamp: 'mg 4/16/2014 11:38'!
initialize

	externalSolver := ClSimplexSolver new.
	constraints := OrderedCollection new.! !

!ConstraintSolver methodsFor: 'as yet unclassified' stamp: 'mg 4/16/2014 13:20'!
solve
	"This method triggers the external solver to solve the constraints. It may raise an exception, if the constraints are unsatisfiable."
	self externalSolver solve.! !

Object subclass: #ExampleClass
	instanceVariableNames: 'a b'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Constraints-Tests'!

!ExampleClass methodsFor: 'accessing' stamp: 'mg 4/15/2014 14:31'!
a

	^ a! !

!ExampleClass methodsFor: 'accessing' stamp: 'mg 4/15/2014 14:31'!
a: anObject

	a := anObject! !

!ExampleClass methodsFor: 'accessing' stamp: 'mg 4/15/2014 14:31'!
b

	^ b! !

!ExampleClass methodsFor: 'accessing' stamp: 'mg 4/15/2014 14:31'!
b: anObject

	b := anObject! !

!ExampleClass methodsFor: 'initialize-release' stamp: 'mg 4/15/2014 14:33'!
initialize

	a := 1.
	b := 2.! !

!Object methodsFor: '*Constraints' stamp: 'mg 4/17/2014 11:02'!
asConstantConstraintVariable

	self error: 'This object cannot be converted to a constant constraint variable because it has no external variable.'.! !

!Object methodsFor: '*Constraints' stamp: 'mg 4/16/2014 13:38'!
asConstraintVariable

	^ ConstraintGenericVariable new originalValue: self! !

!Number methodsFor: '*Constraints' stamp: 'mg 4/17/2014 10:57'!
asConstantConstraintVariable

	^ ConstraintVariable newConstantVariableFor: self! !

!Number methodsFor: '*Constraints' stamp: 'mg 4/16/2014 14:37'!
asConstraintVariable

	^ ConstraintVariable newCassowaryVariableFor: self! !
